// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

// -------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD; //the models world matrix
float4x4 tV: VIEW; //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION; //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture1
texture Tex <string uiname="LiveStream";>;
sampler Samp = sampler_state //sampler for doing the texture-lookup
{
   Texture   = (Tex);          //apply a texture to the sampler
   MipFilter = LINEAR;         //sampler states
   MinFilter = LINEAR;
   MagFilter = LINEAR;
};


//texture2
texture TexBack <string uiname="Background Image";>;
sampler Samp2 = sampler_state //sampler for doing the texture-lookup
{
   Texture   = (TexBack);         //apply a texture to the sampler
   MipFilter = LINEAR;         //sampler states
   MinFilter = LINEAR;
   MagFilter = LINEAR;
};

float thresh = 0.07;

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

struct vs2ps
{
   float4 Pos : POSITION;
   float4 TexCd : TEXCOORD0;
};
// -------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------

vs2ps VS(
   float4 Pos : POSITION,
   float4 TexCd : TEXCOORD0)
{
   //inititalize all fields of output struct with 0
   vs2ps Out = (vs2ps)0;

   //transform position
   Out.Pos = mul(Pos, tWVP);

   //transform texturecoordinates
   Out.TexCd = mul(TexCd, tTex);

   return Out;
}

// -------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------
float4 simpas(vs2ps In): COLOR
{
 float4 col = tex2D(Samp, In.TexCd);//float2(In.TexCd.x+(2.0/320.0),In.TexCd.y));
 return col;
}

float4 bacsub(vs2ps In): COLOR
{
 float4 col = tex2D(Samp, In.TexCd);
 float4 col_back = tex2D(Samp2, In.TexCd);
 float4 tmp = col_back-col;

 if (tmp.r>0) col.r=tmp.r; else  col.r=-tmp.r;
 if (tmp.g>0) col.g=tmp.g; else  col.g=-tmp.g;
 if (tmp.b>0) col.b=tmp.b; else  col.b=-tmp.b;

 return col;
}

float4 bacdiv(vs2ps In): COLOR
{
 float4 col = tex2D(Samp, In.TexCd);
 float4 col_back = tex2D(Samp2, In.TexCd);
 float4 tmp = col / col_back;

 if (tmp.r>0) col.r=tmp.r; else  col.r=-tmp.r;
 if (tmp.g>0) col.g=tmp.g; else  col.g=-tmp.g;
 if (tmp.b>0) col.b=tmp.b; else  col.b=-tmp.b;

 return col;
}

float4 bacdivneg(vs2ps In): COLOR
{
 float4 col = tex2D(Samp, In.TexCd);
 float4 col_back = tex2D(Samp2, In.TexCd);
 float4 tmp = 1 - (col / col_back);

 if (tmp.r>0) col.r=tmp.r; else  col.r=-tmp.r;
 if (tmp.g>0) col.g=tmp.g; else  col.g=-tmp.g;
 if (tmp.b>0) col.b=tmp.b; else  col.b=-tmp.b;

 return col;
}

float4 bacsubthresh(vs2ps In): COLOR
{
 float4 col = tex2D(Samp, In.TexCd);
 float4 col_back = tex2D(Samp2, In.TexCd);
 float4 tmp = col_back-col;
 
 thresh= thresh * thresh * thresh;

 if (tmp.r>0.0) col.r=tmp.r; else  col.r=-tmp.r;
 if (tmp.g>0.0) col.g=tmp.g; else  col.g=-tmp.g;
 if (tmp.b>0.0) col.b=tmp.b; else  col.b=-tmp.b;

 if ((col.r * col.g * col.b)>thresh ) col = 1.0;
 else col= 0.0;
 return col;
}

// -------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------


technique simple_pass //name for the technique pin
{
   pass P0
   {
       VertexShader = compile vs_1_1 VS();
       PixelShader  = compile ps_2_0 simpas();
   }
}

technique Background_subtraction //name for the technique pin
{
   pass P0
   {
       VertexShader = compile vs_1_1 VS();
       PixelShader  = compile ps_2_0 bacsub();
   }
}

technique Background_subtraction_threshold //name for the technique pin
{
   pass P0
   {
       VertexShader = compile vs_1_1 VS();
       PixelShader  = compile ps_2_0 bacsubthresh();
   }
}

technique Background_divide //name for the technique pin
{
   pass P0
   {
       VertexShader = compile vs_1_1 VS();
       PixelShader  = compile ps_2_0 bacdiv();
   }
}

technique Background_divide_neg //name for the technique pin
{
   pass P0
   {
       VertexShader = compile vs_1_1 VS();
       PixelShader  = compile ps_2_0 bacdivneg();
   }
}
